// ExamplePlugin.cpp : Defines the initialization routines for the DLL.
//

#include "ExamplePlugin.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


// constructor of your plugin class
//
ExamplePlugin::ExamplePlugin()
{
	stepCounter = 0;
	nChannels = 0;
}

// destructor of your plugin class
//
ExamplePlugin::~ExamplePlugin()
{
}

bool ExamplePlugin::initPlugin() // this function is called once after class has been created
{
	// This function call and check of return value is necessary to check whether access to TSI works!

	int ret = InitTSIAccess();

	// Never continue, if the "InitTSIAccess" function fails
	//
	if(!ret)
		return false;

	// Test whether the used Turbo-Satori version is new enough to support the commands used in your plugin
	//
	int Major, Minor, SubMinor;
	tGetVersionOfTSI(&Major, &Minor, &SubMinor);

	// We require v0.4.0 or higher
	if( !checkVersionGreaterEqualVersion(Major, Minor, SubMinor, 0, 4, 0) )
	{
		//tMessageBox("Can not execute this plugin - Turbo-Satori version 0.4 or higher required.");
		return false;
	}

	//tMessageBox("Plugin initiated");

	return true;
}

// this is once called before real-time processing is started (Plugin log window is available when this fn is called)
//
bool ExamplePlugin::executePreRun(uint16_t port)
{
	char InfoString[601];
	
	// Test access to TSI and plugins directory:
	char PathToTSI[512];
	tGetTSIPath(PathToTSI);
	sprintf(InfoString, "Info - Path to Turbo-Satori: %s", PathToTSI);
	//tLogText(InfoString);

	char PathToPlugins[512];
	tGetPluginsPath(PathToPlugins);
	sprintf(InfoString, "Info - Path to Plugins: %s", PathToPlugins);
	tLogText(InfoString);
	
	// "tLogText()" sends a line of text to the "Log" text box in the plugin window
	tLogText(" ");
	tLogText("Plugin ready.");
	tLogText(" ");

	stepCounter = 0;
		
	return true;
}

// This is the most important plugin function and is called repeatedly once for every processed time point during real-time processing
//
bool ExamplePlugin::executeTimePoint()
{
	stepCounter++; // a counter of received calls; used here to compare with value obtained via plugin access API 

	char cInfo[601], cVal[41]; // to format log strings

	if(stepCounter == 1) {
		nChannels = tGetNrOfChannels();   // get nr of channels once, store in object var
		sprintf(cInfo, "No. of channels: <b>%i</b>", nChannels);
		tLogText(cInfo);

		char PathToValuesFeedbackFolder[512];
		tGetValuesFeedbackFolder(PathToValuesFeedbackFolder); // get access to default feedback folder (for values) if you want to save data in that folder (subfolder of protocol folder that must be specified)
		sprintf(cInfo, "Path to values feedback folder: %s", PathToValuesFeedbackFolder);
		tLogText(cInfo);

		char PathToImagesFeedbackFolder[512];
		tGetImagesFeedbackFolder(PathToImagesFeedbackFolder);
		sprintf(cInfo, "Path to images feedback folder: %s", PathToImagesFeedbackFolder);
		tLogText(cInfo);
	}

	// access time point just finished processing in TSI
	//
	int tsiTime = tGetCurrentTimePoint(); // 1-based

	// check whether current TBV time matches our own counter:
	if(tsiTime != stepCounter)
		sprintf(cInfo, "Processing unexpected data point \"%i\", expected time point \"%i\"", tsiTime, stepCounter);
	else
		sprintf(cInfo, "Processing data point <b>%i</b>...", tsiTime);
	tLogText(cInfo);

	bool showOxyData = true; // NOTE: if oxy/deoxy concentration not available (baseline not complete or user disabled it), the oxy/deoxy data returns the preprocessed wavelength data
	bool isDataOxyDeoxyConverted = tIsDataOxyDeoxyConverted(); // You can use this call if you want to know if the data at the current time point (frame) is (already) oxy-deoxy converted
	int n_selected_channels = tGetNrOfSelectedChannels();
	int *selChannelList = tGetSelectedChannels();
	float scale_raw_data = tGetRawDataScaleFactor(); // raw values can be quite small - this gives you the scale factor set by the user in the GUI
	float scale_oxy_data = tGetOxyDataScaleFactor(); // converted oxy/deoxy values can be quite small - this gives you the scale factor set by the user in the GUI
	int selChannel;
	float rawVal1, rawVal2;
	sprintf(cInfo, "Currently selected channels: ");
	for(int sel_ch_i=0; sel_ch_i<n_selected_channels; sel_ch_i++) {
		selChannel = selChannelList[sel_ch_i];
		if(showOxyData) {
			rawVal1 =   tGetDataOxy(selChannel, tsiTime - 1) * scale_oxy_data; // -1 since tsiTime 1-based !!
			rawVal2 = tGetDataDeoxy(selChannel, tsiTime - 1) * scale_oxy_data;
		} else {
			rawVal1 = tGetRawDataWL1(selChannel, tsiTime - 1) * scale_raw_data; // -1 since tsiTime 1-based !!
			rawVal2 = tGetRawDataWL2(selChannel, tsiTime - 1) * scale_raw_data;
		}
		sprintf(cVal, "%i: %4.2f %4.2f  ", selChannel + 1, rawVal1, rawVal2);
		strcat(cInfo, cVal);
	}
	tLogText(cInfo);

	return true;
}

// This is once called after real-time processing has been completed (end of scan)
//
bool ExamplePlugin::executePostRun()
{
	tLogText(" ");
	tLogText("Plugin completed processing.");
	
	return true;
}


//
// C access functions (extern "C" in header)
//

// THIS FUNCTION CREATES YOUR PLUGIN CLASS
//
PLUGIN_ACCESS TSIPluginInterface *createPlugin()
{
	return new ExamplePlugin();
}

// return a name for your plugin
//
PLUGIN_ACCESS const char *getPluginName() 
{
	return "Example Plugin";
}

// returns a descriptive string for your plugin
//   it is recommended to use a static C string to describe your plugin
//   (the potentially lengthy string needs then not be created repeatedly in successive calls)
//   Note that you may use simple HTML tags to format the string, such as <b></b>, <i></i> and <br>
//
static const char *PluginDescription = "This simple plugin shows how to access Turbo-Satori data for custom processing at each time step during real-time processing. Use its source code to get started with writing your own plugins.<br>";

PLUGIN_ACCESS const char *getPluginDescription() 
{
	return PluginDescription;
}

// get credit for your work!
//
PLUGIN_ACCESS const char *getAuthor()
{
	return "<i>Rainer Goebel</i>, Brain Innovation B.V.";
}

// you may provide the name for a HTML file containing online help
//
PLUGIN_ACCESS const char *getHelpFile()
{
	return ""; // "ExamplePluginHelp/index.html";
}

// you may provide a version string ("Major.Minor") for (updated) releases of your plugins
//
PLUGIN_ACCESS const char *getPluginVersion()
{
	return "1.0";
}


